% This program verifies the robustness of the method mentioned in our paper "Output Feedback ILC for a Class
%      of Nonminimum Phase Nonlinear Systems with Input Saturation: An Additive-state-decomposition-based Method".
% The original system (24) on page 4 of our paper and our ILC structure are established in "Adjoint_Test8.slx".
% The file "Adjoint_Test8_supp.slx" is used to calculate the adjoint oprator.

clear
clc
close all
switch_Gs = 1;
limax = 3;
limin = -20;

A = [0 1 0; 0 0 1; -4 -6 -4];
B = [0;0;1];
C = [-1 0 1];
D = 0;
% K_Gs = place(A,B,[-4,-4.5,-5]);
% K_Gs = [-1 -5 -1];
K_Gs = [2 5 2];

%% The reference trajectory %%
Ts = 0.01;  % 
TtR = 2*pi;
Tex = 2*pi;
Tt = 6*pi; % չʱ䣬Ϊȶһʱϵ
Nt = floor(Tt/Ts+1);

u_p = zeros(Nt,2);
yd = zeros(Nt,2);
eu = zeros(Nt,2); % ϵͳ
eu2 = zeros(Nt,2);
for i = 1:Nt
    t = (i - 1)*Ts;
    
    u_p(i,1) = t;
    u_p(i,2) = 0;
    eu(i,1) = t;
    eu(i,2) = 0;
    eu2(i,1) = t;
    eu2(i,2) = 0;
    yd(i,1) = t;
    if t < Tex
        yd(i,2) = 0;
    elseif t < Tex + TtR
        yd(i,2) = 1 - cos(t-Tex);
    else
        yd(i,2) = 0;
    end
end

N2 = 20;
e_max1 = zeros(N2,1);
e_max2 = zeros(N2,1);

%% Use the same ILC controller to different delta_phi and delta_A %%
delta_phi = 0;
j = 1;
for delta_A = [-0.2 -0.1 0 0.1 0.2]
    u_p(:,2) = zeros(Nt,1);
    for i = 1:N2
        sim('Adjoint_Test8');
        
        e_max1(i,j) = max(abs(e));
        
        eu(:,2) = flipud(e_p);
        sim('Adjoint_Test8_supp');
        qf = flipud(qb);
        alpha = sum_ek(Nt,1)/sum_Gstar_ek(Nt,1);
        u_p(:,2) = u_p(:,2) + alpha*qf;
        
        clc;
        disp(i);
    end
    j = j + 1;    
end

delta_A = 0;
j = 1;
for delta_phi = [-0.2 -0.1 0 0.1 0.2]
    u_p(:,2) = zeros(Nt,1);
    for i = 1:N2
        sim('Adjoint_Test8');
        
        e_max2(i,j) = max(abs(e));
        
        eu(:,2) = flipud(e_p);
        sim('Adjoint_Test8_supp');
        qf = flipud(qb);
        alpha = sum_ek(Nt,1)/sum_Gstar_ek(Nt,1);
        u_p(:,2) = u_p(:,2) + alpha*qf;
        
        clc;
        disp(i);
    end
    j = j + 1;    
end

%% Plot the results %%
figure(1)
suptitle({'Robustness of the controller'})
set(gcf,'color',[1 1 1])
subplot(1,2,1)
box on
semilogy(1:N2,e_max1(:,1),'k+')
hold on
semilogy(1:N2,e_max1(:,2),'ko')
hold on
semilogy(1:N2,e_max1(:,3),'k*')
hold on
semilogy(1:N2,e_max1(:,4),'ks')
hold on
semilogy(1:N2,e_max1(:,5),'kx')
hold on
legend('delta_A=-0.2','delta_A=-0.1','delta_A=0','delta_A=0.1','delta_A=0.2')
grid on
ylabel('|e|_m_a_x','fontsize',18,'fontname','TimesNewRoman')
xlabel('k','fontsize',18,'fontname','TimesNewRoman')
for i = 1:5
    figure(1)
    subplot(1,2,1)
    semilogy(1:N2,e_max1(:,i),'b')
    hold on
end
title('delta_p_h_i = 0','fontsize',14,'fontname','TimesNewRoman')

figure(1)
set(gcf,'color',[1 1 1])
subplot(1,2,2)
box on
semilogy(1:N2,e_max2(:,1),'k+')
hold on
semilogy(1:N2,e_max2(:,2),'ko')
hold on
semilogy(1:N2,e_max2(:,3),'k*')
hold on
semilogy(1:N2,e_max2(:,4),'ks')
hold on
semilogy(1:N2,e_max2(:,5),'kx')
hold on
legend('delta_phi=-0.2','delta_phi=-0.1','delta_phi=0','delta_phi=0.1','delta_phi=0.2')
grid on
ylabel('|e|_m_a_x','fontsize',18,'fontname','TimesNewRoman')
xlabel('k','fontsize',18,'fontname','TimesNewRoman')
for i = 1:5
    figure(1)
    subplot(1,2,2)
    semilogy(1:N2,e_max2(:,i),'b')
    hold on
end
title('delta_A = 0','fontsize',14,'fontname','TimesNewRoman')


