% This program uses the method proposed in paper "Iterative learning control usingadjoint systems for nonlinear non-minimum phase systems" 
%      to solve the example mentioned in our paper "Output Feedback ILC for a Class of Nonminimum Phase Nonlinear Systems with
%      Input Saturation: An Additive-state-decomposition-based Method".
% The original system (24) on page 4 of our paper is established in "Adjoint_Test6s_nASDB.slx", 
%      and the uncertainties canbe changed in this file.
% The file "Adjoint_Test6s_nASDB_supp.slx" is used to calculate the adjoint oprator.

clear
clc
close all

limax = 3;
limin = -15;

% delta_A = 0;
delta_A = -0.1;

%% The linearized system %%
A = [0 1 0; 0 0 1; -4 -6 -4];
B = [0;0;1];
C = [-1 0 1];
D = 0;
Al = [0 1 0; 0 0 1; -5 -6 -3];

%% The reference trajectory %%
Ts = 0.01;
TtR = 2*pi;
Tex = 2*pi;
Tt = 6*pi;
Nt = floor(Tt/Ts+1);

u_p = zeros(Nt,2);
yd = zeros(Nt,2);
eu = zeros(Nt,2);
eu2 = zeros(Nt,2);
for i = 1:Nt
    t = (i - 1)*Ts;
    
    u_p(i,1) = t;
    u_p(i,2) = 0;
    eu(i,1) = t;
    eu(i,2) = 0;
    eu2(i,1) = t;
    eu2(i,2) = 0;
    yd(i,1) = t;
    if t < Tex
        yd(i,2) = 0;
    elseif t < Tex + TtR
        yd(i,2) = 1 - cos(t-Tex);
    else
        yd(i,2) = 0;
    end
end

%% The ILC method used in paper "Iterative learning control usingadjoint systems for nonlinear non-minimum phase systems" %%
% alpha = {2,5,6} respectively

N2 = 20;
e_max1 = zeros(N2,1);
for i = 1:N2
    sim('Adjoint_Test6s_nASDB');
        
    e_max1(i,1) = max(abs(e));
    
    eu(:,2) = flipud(e_p);
    sim('Adjoint_Test6s_nASDB_supp');
    qf = flipud(qb);

    alpha = 2;
    u_p(:,2) = u_p(:,2) + alpha*qf;
   
    clc;
    disp(i);
end

N2 = 20;
u_p(:,2) = zeros(Nt,1);
e_max2 = zeros(N2,1);
for i = 1:N2
    sim('Adjoint_Test6s_nASDB');
    
    e_max2(i,1) = max(abs(e));

    eu(:,2) = flipud(e_p);
    sim('Adjoint_Test6s_nASDB_supp');
    qf = flipud(qb);
    eu2(:,2) = qf;

    alpha = 5;
    u_p(:,2) = u_p(:,2) + alpha*qf;
    
    clc;
    disp(i);
end

N2 = 20;
u_p(:,2) = zeros(Nt,1);
e_max3 = zeros(N2,1);
for i = 1:N2
    sim('Adjoint_Test6s_nASDB');
    
    e_max3(i,1) = max(abs(e));

    eu(:,2) = flipud(e_p);
    sim('Adjoint_Test6s_nASDB_supp');
    qf = flipud(qb);
    eu2(:,2) = qf;

    alpha = 6;
    u_p(:,2) = u_p(:,2) + alpha*qf;
        
    clc;
    disp(i);
end

figure(1)
set(gcf,'color',[1 1 1])
box on
semilogy(1:N2,e_max1(:,1),'ko')
hold on
semilogy(1:N2,e_max2(:,1),'k+')
hold on
semilogy(1:N2,e_max3(:,1),'k*')
hold on
axis([0,N2,10^-1.2,10^0.8])
ylabel('|e|_m_a_x','fontsize',18,'fontname','TimesNewRoman')
xlabel('k','fontsize',18,'fontname','TimesNewRoman')
legend('alpha = 2','alpha = 5','alpha = 6')
semilogy(1:N2,e_max1(:,1),'b')
hold on
semilogy(1:N2,e_max2(:,1),'b')
hold on
semilogy(1:N2,e_max3(:,1),'b')
hold on
grid on
title({'The convergence process of the controller of "Sogo 2000"'; ['when delta_A = ', num2str(delta_A)]},...
    'fontsize',14,'FontWeight','bold','fontname','TimesNewRoman') 