% This program uses the D-type ILC method to solve the example in our paper.
% The original system (24) on page 4 of our paper and our ILC structure are established in "Adjoint_Test6_classic.slx".
% Notice there is no satuation term in "Adjoint_Test6_classic.slx".

clear
clc
close all

A = [0 1 0; 0 0 1; -4 -6 -4];
B = [0;0;1];
C = [-1 0 1];
D = 0;

%% The reference trajectory %%
Ts = 0.01;
TtR = 2*pi;
Tex = 2*pi;
Tt = Tex*2 + TtR;
Nt = floor(Tt/Ts+1);

u = zeros(Nt,2);
yd = zeros(Nt,2);


for i = 1:Nt
    t = (i - 1)*Ts;
    
    u(i,1) = t;
    u(i,2) = 0;

    yd(i,1) = t;
    if t < Tex
        yd(i,2) = 0;
    elseif t < Tex + TtR
        yd(i,2) = 1 - cos(t-Tex);
    else
        yd(i,2) = 0;
    end
end

%% The D-type ILC method %%
figure(1)
set(gcf,'color',[1 1 1])
subplot(2,1,1)
plot(u(:,1),yd(:,2))
ylabel('y_k','fontsize',18,'fontname','TimesNewRoman')
xlim([0 Tt])
hold on
title({'The convergence process of y_k by using D-type ILC'; '(The darker the line is, the larger number k is.)'},...
    'fontsize',14,'FontWeight','bold','fontname','TimesNewRoman')
subplot(2,1,2)
plot(u(:,1),yd(:,2))
ylabel('y_k in detail','fontsize',18,'fontname','TimesNewRoman')
xlabel('t/s','fontsize',18,'fontname','TimesNewRoman')
axis([0 Tt -5.5 5.5])
hold on

j = 0;
p = 1;
N1 = 44;
for i = 1:N1
    sim('Adjoint_Test6_classic');
        
    if i == p
        figure(1)
        subplot(2,1,1)
        plot(u(:,1),y(:,1),'color',[1-(i+22)/(N1+22),1-(i+22)/(N1+22),1-(i+22)/(N1+22)])
        subplot(2,1,2)
        plot(u(:,1),y(:,1),'color',[1-(i+22)/(N1+22),1-(i+22)/(N1+22),1-(i+22)/(N1+22)])
        j = j + 1;
        p = p + 2*j;
    end
    
    u(:,2) = u(:,2) + 1.1*de;
    
    clc;
    disp(i);
end
%%%%%%%%%%%%%%%%

